/*****************************************************************************

**      **   **
****  ****    **
** **** **     **
**  **  **      **      $RCSfile: cvviewer.C,v $
**      **     **
**      **    **
**      **   **

	By:	Jon Goldman
	
	Date:
	
Modification Date:  $Date: 1995/07/21 00:31:22 $
	
Description:

Revisions:


*****************************************************************************/

/***** include files *****/
#include <stdlib.h>
#include <iostream.h>

#include "cvviewer.h"

/***** macros or const (c++) *****/

/***** function prototypes *****/

/***** typedefs or classes (c++) *****/

/***** storage *****/
float CvViewer::globalT[3]={0.0,0.0,0.0};
float CvViewer::globalMin[3]={0.0,0.0,0.0};
float CvViewer::globalMax[3]={0.0,0.0,0.0};
float CvViewer::TRANS_FACTOR=.250;

/**********************************************************************************************
**************************************** Start of code ****************************************
**************************************** Start of code ****************************************
**************************************** Start of code ****************************************
**************************************** Start of code ****************************************
***********************************************************************************************/


DatabaseList::DatabaseList(char *filename)
{
    void BuildInventorScene(char *filename, SoSeparator *root);
    const float *center;
    const float *min;
    const float *max;
    
    cerr << "DatabaseList reading Inventor file: " << filename << endl;
    root = new SoSeparator;
    root->ref();
    root->setName(SbName(filename));
    BuildInventorScene(filename, root);

    SbViewportRegion vpReg;		// needed for SoGetBoundingBoxAction constructor
    vpReg.setWindowSize(0, 0);
    
    SoGetBoundingBoxAction ba(vpReg);
    ba.apply(root);
    bbox = ba.getBoundingBox();
    center = bbox.getCenter().getValue();

    cerr << "Center of database read from file " << filename <<
      " is (" << center[0] << ", " << center[1] << ", " << center[2] << ") " << endl;

    min = bbox.getMin().getValue();
    max = bbox.getMax().getValue();

    int ii;
    for (ii=0; ii<3; ii++)
      {
	  CvViewer::globalMin[ii] = MIN(CvViewer::globalMin[ii],min[ii]);
	  CvViewer::globalMax[ii] = MAX(CvViewer::globalMax[ii],max[ii]);
	  CvViewer::globalT[ii] += center[ii];
      }

    next = NULL;
}


CvViewer::CvViewer(int argc, char *argv[])
{
    int argCount = argc;
    numFiles = 0;
    float maxExtent=0.0;
    int ii;
    
    // read in the inventor database
    
    // Initialize Inventor scene graph database. required before calling any
    // other database routines, and before constructing any nodes, paths, functions, or actions.
    SoDB::init();

    if (argc < 2)
      {
	  cerr << "Usage:	cvview  filename ..." << endl << endl;
	  exit(1);
      }
    else
      {
	  DatabaseList **dbl = &database;
	  numFiles = argCount-1;
	  int ii=0;
	  
    	  while (--argCount)
	    {
		ii++;
		char fileName[2048];
		strcpy(fileName, argv[argCount]);
		cerr << "CvViewer reading number " << ii << " Inventor file: " << fileName << endl;
		*dbl = new DatabaseList(fileName);
		dbl = &((*dbl)->next);
	    }
      }

    
    // make a point light source
    pointLight = new SoPointLight();
    pointLight->ref();		// add a reference so that it doesn't get deleted outside this routine

    // put point light at the center of the cave
    pointLight->intensity.setValue(0.6);	/* less bright than maximum */
    pointLight->location.setValue(0.0,5.0,0.0);	// bad bad, hardcoded cave center...now where did i put those pesky cave coordinates...

    spotLight = new SoSpotLight();
    spotLight->ref();          // add a reference so that it doesn't get deleted outside this routine
    spotLight->dropOffRate.setValue(0.0);	/* 0.0=constant intensity, no dropoff */
    spotLight->cutOffAngle.setValue(M_PI/10.0);	/* angle of light beam */

    // adjust the global Transformations
    for (ii=0; ii<3; ii++)
      {
	  maxExtent = MAX(CvViewer::globalMax[ii] - CvViewer::globalMin[ii], maxExtent);
	  CvViewer::globalT[ii] /= -1.0*numFiles;
      }

    cerr << "Extent box ranges from min: " <<
      "(" << CvViewer::globalMin[0] << ", " << CvViewer::globalMin[1] << ", " << CvViewer::globalMin[2] << ") " << "  to  max: : " <<
	"(" << CvViewer::globalMax[0] << ", " << CvViewer::globalMax[1] << ", " << CvViewer::globalMax[2] << ") " << endl;

    // Adjust the TRANS_FACTOR appropriately. Assume about 25 frames per second rendering
    // and that one would want to cover the entire scene end-to-end within 3 seconds
    cerr << "Maximum extent is " << maxExtent << endl;
    CvViewer::TRANS_FACTOR = maxExtent/(3*25.0);
    cerr << "TRANS_FACTOR is " << CvViewer::TRANS_FACTOR << endl;

    cerr << "default global translation alignment = ("<<
      CvViewer::globalT[0] << ", " << CvViewer::globalT[1] << ", " << CvViewer::globalT[2] << ") " << endl;
    myTrans = new SoTransform;
    myTrans->translation.setValue(CvViewer::globalT[0],CvViewer::globalT[1],CvViewer::globalT[2]);
    myTrans->ref();          // add a reference so that it doesn't get deleted outside this routine

    navRot = new SoRotation;
    navRot->ref();	// add a reference so that it doesn't get deleted outside this routine

    mode = new Mode(numFiles);
    
    interface = new CaveInterface();
}
    
